﻿using NLScC.model;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;

namespace NLScC.Command
{

    public static class ManagedCommands
    {
        public static ICommand OpenMemberAdministrationCommand { get; set; }
        public static ICommand OpenPhonebookCutterCommand { get; set; }
        public static ICommand ExportConsoleCommand { get; internal set; }
        public static ICommand ToolAdministrationCOmmand { get; internal set; }
        public static ICommand ManualUpdateCommand { get; internal set; }
        public static ICommand ProgramUpdateCommand { get; internal set; }
        public static ICommand EmergencySetupCommand { get; internal set; }
        public static ICommand StartWithWindowsCommand { get; internal set; }

        static ManagedCommands()
        {
            OpenMemberAdministrationCommand = new RelayCommand(OpenMemberAdministrationExecute);
            OpenPhonebookCutterCommand = new RelayCommand(OpenPhonebookCutter);
            ExportConsoleCommand = new RelayCommand(ExportConsoleExecute);
            ToolAdministrationCOmmand = new RelayCommand(ToolAdminsistrationExecute);
            ManualUpdateCommand = new RelayCommand(ManualUpdateExecute);
            ProgramUpdateCommand = new RelayCommand(ProgramUpdateExecute);
            EmergencySetupCommand = new RelayCommand(EmergencySetupExecute);
            StartWithWindowsCommand = new RelayCommand(StartWithWindowsExecute);
        }



        private static SimpleWifi.Wifi WIFI;

        private async static void EmergencySetupExecute(object obj)
        {
            WIFI = new SimpleWifi.Wifi();
            LogConsole.Log("Emergency SetUpStartet:", LogConsole.LogLevel.Warning);
            LogConsole.Log("WiFi-Connection to M-Gas", LogConsole.LogLevel.Warning);
            LogConsole.Log("Connection will be established", LogConsole.LogLevel.Warning);
            LogConsole.Log("Internal URL for ASSP will be loaded immediatly", LogConsole.LogLevel.Warning);
            List<SimpleWifi.AccessPoint> accessPoints = WIFI.GetAccessPoints();
            foreach (SimpleWifi.AccessPoint ap in accessPoints)
            {
                if (ap.Name == "M-Gas")
                {
                    if (ConnectToWLAN(ap)) break;
                }
            }

            await Task.Delay(10000);
            LogConsole.Log("Connection Established!", LogConsole.LogLevel.Warning);
            LogConsole.Log("DO NOT FORGET TO RESET TO NORMAL-MODE AFTERWARDS!", LogConsole.LogLevel.Warning);
            try
            {
                await Task.Run( () => StartEmergencyURL());
            }
            catch(Exception e)
            {
                LogConsole.Log("Notfall-Gateway <https://assp4int-ava.supinfo.de> konnte nicht geöffnet werden: " + e.Message);
            }
        }

        private static void StartEmergencyURL()
        {
            System.Diagnostics.Process process = System.Diagnostics.Process.Start("https://assp4int-ava.supinfo.de");
        }

        private static bool ConnectToWLAN(SimpleWifi.AccessPoint ap)
        {
            SimpleWifi.AuthRequest authRequest = new SimpleWifi.AuthRequest(ap);
            authRequest.Password = "AVA-Meldestelle-2028";

            return ap.Connect(authRequest);
        }

        private static void ProgramUpdateExecute(object obj)
        {
            try
            {
                string newVersion = "";
                using (System.IO.StreamReader reader = new System.IO.StreamReader(@"G:\TEAM\EAV\V-N\NT\NTS\NTS-R\05_Kollektiv\Programme\NLSCC\Release\latest\version.txt"))
                {
                    newVersion = reader.ReadToEnd();
                }
                Process process = Process.GetCurrentProcess();
                string args = AppDomain.CurrentDomain.BaseDirectory + " " + AppSettings.Settings.Version + " " + newVersion + " " + process.Id;
                Process.Start(@"G:\TEAM\EAV\V-N\NT\NTS\NTS-R\05_Kollektiv\Programme\NLSCC\Release\latest\NLScCUpdate.exe", args);
            }
            catch (Exception ex)
            {
                if (ex is System.IO.FileNotFoundException || ex is System.IO.DirectoryNotFoundException)
                {
                    LogConsole.Log("Auf das Updatesystem konnte aufgrund von Ordner-Problemen nicht zugegriffen werden!", LogConsole.LogLevel.Warning);
                }             
                else          
                {             
                    LogConsole.Log("Error_Code_0x001: Message" + ex.Message,LogConsole.LogLevel.Error);
                    LogConsole.Log("Error_Code_0x001: Stack" + ex.StackTrace, LogConsole.LogLevel.Error);
                    LogConsole.Log("Error_Code_0x001: Source" + ex.Source, LogConsole.LogLevel.Error);
                    LogConsole.Log("Error_Code_0x001: InnerEx" + ex.InnerException, LogConsole.LogLevel.Error);
                }
            }
        }

        private static void ManualUpdateExecute(object obj)
        {
            string phonebookFromPath = AppSettings.PhonebookTeamPath;
            string memberFromPath = AppSettings.MemberTeamPath;
            string phonbookToPath = AppSettings.PhonebookFilePath;
            string memberToPath = AppSettings.MemberFilePath;
            try
            {
                System.IO.File.Copy(phonebookFromPath, phonbookToPath, true);
                System.IO.File.Copy(memberFromPath, memberToPath, true);
                LogConsole.Log("Telefonbuch- und Member-Files erfolgreich manuell kopiert. Diese sind nach einem Neustart des Programms verfügbar");
            }
            catch (System.IO.FileNotFoundException)
            {
                LogConsole.Log("Die Master-Files konnten nicht gefunden werden. Scheinbar wurden sie gelöscht. Bitte wenden Sie sich an den Administrator", LogConsole.LogLevel.Warning);
            }
            catch (System.IO.DriveNotFoundException)
            {
                LogConsole.Log("Aif das Teamlaufwerk konnte nicht zugegriffen werden. Eventuell bestehen derzeit Zugriffsprobleme - Dann versuchen Sie es später erneut. Im Homeoffice: Bitte erst das LogOn-Script ausführen", LogConsole.LogLevel.Warning);
            }
            catch (Exception e)
            {
                LogConsole.Log("Schwerwiegender Fehler: " + e.Message, LogConsole.LogLevel.Error);
            }
        }

        private static void ToolAdminsistrationExecute(object obj)
        {
            throw new NotImplementedException();
        }

        private static void ExportConsoleExecute(object obj)
        {
            var t = LogConsole.Logger;
            //string path = System.IO.Path.Combine( Assembly.GetExecutingAssembly().Location , "logs");
            string path = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "logs");
            string filename = path + @"\LOGS_" + DateTime.Now.ToString("HHmmss") + ".rtf";
            if (!System.IO.Directory.Exists(path)) System.IO.Directory.CreateDirectory(path);
            if (!System.IO.File.Exists(filename)) System.IO.File.Create(filename).Close();
            using (System.IO.FileStream stream = new System.IO.FileStream(filename, System.IO.FileMode.OpenOrCreate))
            {
                t.SelectAll();
                t.Selection.Save(stream, dataFormat: DataFormats.Rtf);
            }
        }

        private static void OpenPhonebookCutter(object obj)
        {
            Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog()
            {
                Filter = "Text-File|*.txt",
                InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory),
                Title = "PhonebookCutter - Datei auswählen"
            };
            if (dialog.ShowDialog() == true)
            {
                NLScC.model.PhoneEntryCollection collection = NLScC.FileConverter.PhonebookCutter.ConvertTextFileToPhonebookCollection(dialog.FileName);
                NLScC.model.Writer.WriteFile(NLScC.model.AppSettings.PhonebookTeamPath, collection, "phone");
            }
        }

        private static void OpenMemberAdministrationExecute(object obj)
        {
            NLScC.dialog.MemberAdministrationDialog dialog = new dialog.MemberAdministrationDialog()
            {
                Owner = MainWindow.AppReference,
                Top = MainWindow.AppReference.Top + 10,
                Left = MainWindow.AppReference.Left + 30,
                FontSize = MainWindow.AppReference.FontSize,
                FontFamily = MainWindow.AppReference.FontFamily
            };
            dialog.ShowDialog();
        }



        #region StartUp

        private static void StartWithWindowsExecute(object obj)
        {
            CreateStartUpAutostart();
        }

        private static void CreateStartUpAutostart()
        {
            if (AppSettings.Settings.IsStartetWithWindows)
            {
                string PathAutostart = Environment.GetFolderPath(Environment.SpecialFolder.Startup);
                if (System.IO.File.Exists(PathAutostart + @"\NLScCAutostarter_v1.lnk")) System.IO.File.Delete(PathAutostart + @"\NLScCAutostarter_v1.lnk");
                string AutoStartFile = PathAutostart + @"\NLScCAutostarter_v2.lnk";
                //if (System.IO.File.Exists(AutoStartFile)) return;
                IWshRuntimeLibrary.WshShell wsc = new IWshRuntimeLibrary.WshShell();
                IWshRuntimeLibrary.IWshShortcut shortcut = wsc.CreateShortcut(AutoStartFile);
                shortcut.Arguments = "WindowsAutostartTrue";
                shortcut.TargetPath = AppDomain.CurrentDomain.BaseDirectory + @"\NLScc.exe";
                shortcut.WindowStyle = 1;
                shortcut.Description = "A little Shortcut to start the App on Windows Startup.";
                shortcut.WorkingDirectory = AppDomain.CurrentDomain.BaseDirectory;
                shortcut.Save();
            }
        }

        #endregion


    }
}
