﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;
using NLScC.model;
using NLScC.Command;

namespace NLScC.ViewModel
{
    public class PhonebookViewModel : NotifyBase
    {
        private string _filterLastname;
        public string FilterLastname
        {
            get { return _filterLastname; }
            set { SetField(ref _filterLastname, value); ApplyFilters(); }
        }

        private string _filterPrename;
        public string FilterPrename
        {
            get { return _filterPrename; }
            set { SetField(ref _filterPrename, value); ApplyFilters(); }
        }

        private string _filterCompany;
        public string FilgerCompany
        {
            get { return _filterCompany; }
            set { SetField(ref _filterCompany, value); ApplyFilters(); }
        }

        private string _filterUnit;
        public string FilterUnit
        {
            get { return _filterUnit; }
            set { SetField(ref _filterUnit, value); ApplyFilters(); }
        }

        private string _filterPhonenumber;
        public string FilterPhonenumber
        {
            get { return _filterPhonenumber; }
            set { SetField(ref _filterPhonenumber, value); ApplyFilters(); }
        }

        private string _filterKID;
        public string FilterKID
        {
            get { return _filterKID; }
            set { SetField(ref _filterKID, value); ApplyFilters(); }
        }

        private string _filterAdress;
        public string FilterAdress
        {
            get { return _filterAdress; }
            set { SetField(ref _filterAdress, value); ApplyFilters(); }
        }

        private string _filterSupervisor;
        public string FilterSupervisor
        {
            get { return _filterSupervisor; }
            set { SetField(ref _filterSupervisor, value); ApplyFilters(); }
        }

        private void ApplyFilters()
        {
            _phonebookView.Filter = Filtering;
        }

        private bool Filtering(object obj)
        {
            var s = obj as PhonebookEntry;
            return Contains(s.Lastname, FilterLastname) && Contains(s.Prename, FilterPrename) && Contains(s.Company, FilgerCompany) &&
                Contains(s.Unit, FilterUnit) && Contains(s.KID, FilterKID) && Contains(s.Supervisor, FilterSupervisor) && ContainsPhonenumber(s) && ContainsAddress(s);
        }

        private bool Contains(string field, string filter)
        {
            if (filter == null || string.IsNullOrWhiteSpace(filter)) return true;
            return field.ToLower().Contains(filter.ToLower());
        }

        private bool ContainsPhonenumber(PhonebookEntry e)
        {
            if (FilterPhonenumber == null || string.IsNullOrWhiteSpace(FilterPhonenumber)) return true;
            return e.Phone1.Replace(" ", "").Replace("-", "").Replace("/", "").ToLower().Contains(FilterPhonenumber) || e.Phone2.Replace(" ", "").Replace("-", "").Replace("/", "").ToLower().Contains(FilterPhonenumber) ||
                e.Mobile.Replace(" ", "").Replace("-", "").Replace("/", "").ToLower().Contains(FilterPhonenumber) || e.Private1.Replace(" ", "").Replace("-", "").Replace("/", "").ToLower().Contains(FilterPhonenumber)
                || e.Private2.Replace(" ", "").Replace("-", "").Replace("/", "").ToLower().Contains(FilterPhonenumber);
        }

        private bool ContainsAddress(PhonebookEntry e)
        {
            if (FilterAdress == null || string.IsNullOrWhiteSpace(FilterAdress)) return true;
            string s = FilterAdress.ToLower();
            return e.City.ToLower().Contains(s) || e.Street.ToLower().Contains(s) || e.Country.ToLower().Contains(s);
        }


        private PhoneEntryCollection _phonebookEntries;
        private ListCollectionView _phonebookView;
        public ListCollectionView PhonebookView
        {
            get { return _phonebookView; }
        }


        public PhonebookViewModel()
        {
            _phonebookEntries = new PhoneEntryCollection();
            LoadPhonebook();
            //LoadTestData();
            _phonebookView = new ListCollectionView(_phonebookEntries);
            _phonebookView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Lastname", System.ComponentModel.ListSortDirection.Ascending));
            _phonebookView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Prename", System.ComponentModel.ListSortDirection.Ascending));
            _phonebookView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Company", System.ComponentModel.ListSortDirection.Ascending));
            _phonebookView.SortDescriptions.Add(new System.ComponentModel.SortDescription("Unit", System.ComponentModel.ListSortDirection.Ascending));
            _phonebookView.SortDescriptions.Add(new System.ComponentModel.SortDescription("KID", System.ComponentModel.ListSortDirection.Ascending));
        }

        private void LoadPhonebook()
        {
            if (System.IO.File.Exists(AppSettings.PhonebookFilePath))
            {

                try
                {
                    PhoneEntryCollection c = (PhoneEntryCollection)Reader.ReadFile(AppSettings.PhonebookFilePath, _phonebookEntries, DateTime.Now.ToString("HHmmssffff"));
                    foreach (PhonebookEntry e in c)
                    {
                        _phonebookEntries.Add(e);
                    }
                    LogConsole.Log("Phonebook-Files erfolgreich geladen");
                }
                catch (Exception e)
                {
                    string message = "Beim Einlesen des Telefonbuchs ist ein schwerer Fehler aufgetreten. Bitte diesen Umstand mit folgender Fehlermeldung melden: " + Environment.NewLine + e.Message;
                    LogConsole.LogLevel lvl = LogConsole.LogLevel.Error;
                    LogConsole.Log(message, lvl);
                }
            }
            else
            {
                string message = "Die Offline-Kopie des Telefonbuchs ist nicht vorhanden. Erwägen Sie einen manuellen Updatevorgang und starten Sie danach das Programm neu";
                LogConsole.LogLevel lvl = LogConsole.LogLevel.Warning;
                LogConsole.Log(message, lvl);
            }
        }



        private void FillPhonebook()
        {

        }

        private void LoadTestData()
        {
            _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon AG", "NTSR", "0170 3698413", "", "", "", "", "Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy2", "Vorname", "Avacon AG", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy3", "Vorname", "Avacon AG", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy4", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy5", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("Shlumpi", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("Kocki", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("Impu", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("Tzuni", "Hans", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "G918", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("KaiCho", "Jörg", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "P31931", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("Loppi", "Kopp", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "D120821", "test@mail", "Großer Mann"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Lutschi", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "U18122", "test@mail", "Kleiner Mann"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "O827176", "test@mail", "Dicker Mann"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T1321", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "BU", "0170 35984531", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "KI", "0170 33338844", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "OP", "0176 0654641", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "OP", "080080660851", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "OP", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
           _phonebookEntries.Add(new PhonebookEntry("TestDummy", "Vorname", "Avacon Netz GmbH", "NTSR", "0192 82172151", "","","","","Watenstedter Weg 75", "Salzgitter", "Deutschland", "T29187", "test@mail", "TestSupervisor"));
        }
    }
}
