﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using NLScC.model;

namespace NLScC.dialog
{
    /// <summary>
    /// Interaktionslogik für AddWebDialog.xaml
    /// </summary>
    public partial class AddWebDialog : Window
    {

        public ApplicationType WebType { get; set; }

        public AddWebDialog()
        {
            InitializeComponent();
            WebType = ApplicationType.Website;
        }

        public AddWebDialog(bool useGroupBox, List<string> groupList) : this()
        {
            if (useGroupBox)
            {
                GroupBox.Visibility = Visibility.Visible;
                GroupInsert.Visibility = Visibility.Visible;
                GroupInserText.Visibility = Visibility.Visible;
                for (int i = 0; i < groupList.Count; i++)
                {
                    GroupBox.Items.Add(groupList.ElementAt(i));
                }
                GroupBox.SelectedIndex = 0;
            }
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            if(!string.IsNullOrEmpty(WebName.Text) && !string.IsNullOrEmpty(WebURL.Text))
            {
                ExtractFavIcon(WebURL.Text);
                this.DialogResult = true;
            }
        }

        public void ExtractFavIcon(string URL)
        {
            string CutUrl = URL.Replace("http:", "").Replace("https://", "").Replace("www.", "");
            if (CutUrl.Contains("/")) CutUrl = CutUrl.Substring(0, CutUrl.IndexOf("/"));


            try
            {
                if (MainWindow.WebDictionary.ContainsKey(CutUrl)) return;
                var client = new System.Net.WebClient();
                string managedURL = @"https://www.google.com/s2/favicons?sz=64&domain=" + URL;
                client.DownloadFile(managedURL, AppSettings.FavIconPath + @"\" + CutUrl + ".png");
                MainWindow.WebDictionary.Add(CutUrl, AppSettings.FavIconPath + @"\" + CutUrl + ".png");
            }
            catch (System.Net.WebException e)
            {
                MessageBox.Show("Ein Fehler beim Hinzufügen einer Webseite ist aufgetreten:" + Environment.NewLine + Environment.NewLine + e.Message);
            }
            catch(Exception ex)
            {
                MessageBox.Show("Ein allgemeiner Fehler beim Hinzufügen einer Webseite ist aufgetreten. Näheres entnehmen Sie bitte der angehängten Fehlermeldung und reichen diese weiter:" + Environment.NewLine + Environment.NewLine + ex.Message);
            }
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            this.DialogResult = false;
        }

        private void RadioButton_Checked(object sender, RoutedEventArgs e)
        {
            string s = (sender as RadioButton).Content.ToString();
            switch (s)
            {
                case "Chrome":
                    WebType = ApplicationType.Chrome;
                    break;
                case "IE / Edge":
                    WebType = ApplicationType.Edge;
                    break;
                case "Firefox":
                    WebType = ApplicationType.Firefox;
                    break;
                default:
                    WebType = ApplicationType.Website;
                    break;
            }
        }

        private void GroupBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            GroupInsert.Text = GroupBox.SelectedItem.ToString();
        }
    }
}
