﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Drawing;
using System.Windows.Interop;
using System.Windows;
using System.Runtime.Serialization;

namespace NLScC.model
{
    [Serializable]
    public class ManagedApplication : NotifyBase
    {
        private string _path;
        public string Path
        {
            get { return _path; }
            set { SetField(ref _path, value); }
        }

        private string _args;
        public string Args
        {
            get { return _args; }
            set { SetField(ref _args, value); }
        }

        private string _webIconSource;
        public string WebIconSource
        {
            get { return _webIconSource; }
            set { SetField(ref _webIconSource, value); }
        }

        private string _imageIconSource;
        public string ImageIconSource
        {
            get { return _imageIconSource; }
            set { SetField(ref _imageIconSource, value); }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
            set { SetField(ref _description, value); }
        }

        private ApplicationType _appType;
        public ApplicationType AppType
        {
            get { return _appType; }
            set { SetField(ref _appType, value);
                SetImages();
            }
        }

        private string _group;
        public string Group
        {
            get { return _group; }
            set { SetField(ref _group, value); }
        }

        private bool _expanded;
        public bool Expanded
        {
            get { return _expanded; }
            set { SetField(ref _expanded, value); }
        }

        public ManagedApplication(ApplicationType appType, string description, string path,string group = "Not Grouped", string args = "", string webSource = "", bool expanded = false)
        {
            this._description = description;
            this._appType = appType;
            this._path = path;
            this._args = args;
            this._group = group;
            this._imageIconSource = @"pack://application:,,,/images/other/404.png";
            this._expanded = expanded;

            SetImages();
        }


        private void SetWebIcon()
        {
            switch (_appType)
            {         
                case ApplicationType.Chrome:
                    this.WebIconSource = @"pack://application:,,,/images/web/chrome.png";
                    break;
                case ApplicationType.Firefox:
                    this.WebIconSource = @"pack://application:,,,/images/web/firefox.png";
                    break;
                case ApplicationType.Edge:
                    this.WebIconSource = @"pack://application:,,,/images/web/edge.png";
                    break;
                case ApplicationType.Website:
                    this.WebIconSource = @"pack://application:,,,/images/web/web.png";
                    break;
                default:
                    this._webIconSource = null;
                    this._imageIconSource = @"pack://application:,,,/images/other/404.png";
                    break;
            }
        }

        private void SetImages()
        {
            switch (_appType)
            {
                case ApplicationType.Application:
                    SetWebIcon();
                    this._imageIconSource = _path;
                    break;
                case ApplicationType.Chrome:
                    SetWebIcon();
                    this._imageIconSource = GetImageFromDirectory(_path);
                    break;
                case ApplicationType.Firefox:
                    SetWebIcon();
                    this._imageIconSource = GetImageFromDirectory(_path);
                    break;
                case ApplicationType.Edge:
                    SetWebIcon();
                    this._imageIconSource = GetImageFromDirectory(_path);
                    break;
                case ApplicationType.Website:
                    SetWebIcon();
                    this._imageIconSource = GetImageFromDirectory(_path);
                    break;
                case ApplicationType.Folder:
                    SetWebIcon();
                    this._imageIconSource = @"pack://application:,,,/images/programmanager/foldermanager.png";
                    break;
                default:
                    this._webIconSource = @"pack://application:,,,/images/others/404.png";
                    this._imageIconSource = @"pack://application:,,,/images/others/404.png";
                    break;
            }
        }

       
        private string GetImageFromDirectory(string URL)
        {
            try
            {
                string CutUrl = URL.Replace("http:", "").Replace("https://", "").Replace("www.", "");
                if (CutUrl.Contains("/")) CutUrl = CutUrl.Substring(0, CutUrl.IndexOf("/"));

                return MainWindow.WebDictionary[CutUrl];
            }
            catch (KeyNotFoundException)
            {
                string message = "Key in Dictionary not Found on: " + URL;
                LogConsole.LogLevel lvl = LogConsole.LogLevel.Info;
                if (LogConsole.Logger != null)
                {
                    LogConsole.Log(message,lvl);
                }
                else
                {
                    LogConsole.ErrorList.Add(new LogConsole.Errors(message,lvl));
                }
                return @"pack://application:,,,/images/other/404.png";
            }
            catch (Exception e)
            {
                string message = "General Exception due to assining Key from WebDictionary: " + Environment.NewLine + "Error:  " + e.Message;
                LogConsole.LogLevel lvl = LogConsole.LogLevel.Warning;
                if (LogConsole.Logger != null)
                {
                    LogConsole.Log(message,lvl);
                }
                else
                {
                    LogConsole.ErrorList.Add(new LogConsole.Errors(message, lvl));
                }
                return @"pack://application:,,,/images/other/404.png";
            }
        }

    }
}
